/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.common.xml.XMLHelper;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class IntelliFormMapper {

	public FormData mapToFormData(DepositData depositData) {
		// Transform deposit data attachments to incoming files
		Map<String, IncomingFile> incomingFileMap = depositData.getAttachments()
				.stream()
				.collect(
						Collectors.toMap(Attachment::getId, this::mapAttachmentToIncomingFile)
				);

		// Parse the primary <myForm> xml representation
		Document document = XMLHelper.parseDocument(
				// Expect that the <primaryDataAttachmentId> refers to the XML file (and the <primaryFormAttachmentId> to the PDF file)
				getIncomingFileById(depositData.getPrimaryDataAttachmentId(), incomingFileMap)
		);

		// Find ids of <file> attachments
		var attachmentFileIds = findAttachmentIds(document);
		var attachments = attachmentFileIds.stream()
				.map(id -> getIncomingFileById(id, incomingFileMap))
				.map(this::createSingularFileGroup)
				.toList();
		var representations = getNamesWithout(incomingFileMap.keySet(), attachmentFileIds).stream()
				.map(id -> getIncomingFileById(id, incomingFileMap))
				.toList();
		return FormData.builder()
				// Put each <file> attachment in a separate file group
				.attachments(attachments)
				.numberOfAttachments(attachments.size())
				// If a deposit attachment is not referenced by <file> it is interpreted as a representation
				.representations(representations)
				.numberOfRepresentations(representations.size())
				.build();
	}

	private IncomingFile getIncomingFileById(String id, Map<String, IncomingFile> incomingFileMap) {
		if (!incomingFileMap.containsKey(id)) {
			throw new TechnicalException("Failed to find deposit data attachment by id '%s'!".formatted(id));
		}
		return incomingFileMap.get(id);
	}

	private IncomingFileGroup createSingularFileGroup(IncomingFile incomingFile) {
		return IncomingFileGroup.builder().file(incomingFile).build();
	}

	private List<String> getNamesWithout(Collection<String> names, Collection<String> excludedStrings) {
		var excludedStringsSet = Set.copyOf(excludedStrings);
		return names.stream().filter(name -> !excludedStringsSet.contains(name)).toList();
	}

	private List<String> findAttachmentIds(Document document) {
		return streamElements(document.getElementsByTagName("file"))
				.map(element -> element.getAttribute("id"))
				.toList();
	}

	private Stream<Element> streamElements(NodeList nodeList) {
		return IntStream.range(0, nodeList.getLength())
				.mapToObj(nodeList::item)
				.map(Element.class::cast);
	}

	IncomingFile mapAttachmentToIncomingFile(Attachment attachment) {
		var file = TempFileUtils.writeTmpFile(attachment.content);
		return IncomingFile.builder()
				.id(UUID.randomUUID().toString())
				.vendorId(attachment.id)
				.name(attachment.name)
				.contentType(attachment.contentType)
				.size(file.length())
				.file(file)
				.build();
	}
}
