/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import static de.ozgcloud.eingang.intelliform.AttachmentTestFactory.*;
import static de.ozgcloud.eingang.intelliform.DepositDataTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.nio.charset.Charset;
import java.util.Collections;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import lombok.SneakyThrows;

class IntelliFormMapperTest {

	@Spy
	private IntelliFormMapper mapper;

	@DisplayName("map to form data")
	@Nested
	class TestMapToFormData {

		@Captor
		private ArgumentCaptor<Attachment> attachmentArgumentCaptor;

		private DepositData depositData;

		@DisplayName("with normal attachments")
		@Nested
		class TestWithNormalAttachments {
			@BeforeEach
			void mock() {
				depositData = DepositDataTestFactory.create(ATTACHMENTS);
			}

			@DisplayName("should throw technical exception if primary xml link is incorrect")
			@Test
			void shouldThrowTechnicalExceptionIfPrimaryXmlLinkIsIncorrect() {
				depositData.setPrimaryDataAttachmentId("incorrect");

				assertThatThrownBy(TestMapToFormData.this::doMapping)
						.isInstanceOf(TechnicalException.class);
			}

			@DisplayName("should use map to incoming file method")
			@Test
			void shouldUseMapToIncomingFileMethod() {
				doMapping();
				verify(mapper, times(ATTACHMENTS.size())).mapAttachmentToIncomingFile(attachmentArgumentCaptor.capture());

				assertThat(attachmentArgumentCaptor.getAllValues()).isEqualTo(ATTACHMENTS);
			}

			@DisplayName("should return with representations")
			@Test
			void shouldReturnWithRepresentations() {
				var formData = doMapping();

				var incomingFileIds = formData.getRepresentations().stream().map(IncomingFile::getVendorId).toList();
				assertThat(incomingFileIds).contains(XML_ATTACHMENT_ID, PDF_ATTACHMENT_ID);
			}

			@DisplayName("should return with attachments")
			@Test
			void shouldReturnWithAttachments() {
				var formData = doMapping();

				var incomingFileIds = formData.getAttachments().stream()
						.flatMap(group -> group.getFiles().stream())
						.map(IncomingFile::getVendorId)
						.toList();
				assertThat(incomingFileIds).contains(PNG_ATTACHMENT_ID);
			}

			@DisplayName("should return with number of representations")
			@Test
			void shouldReturnWithNumberOfRepresentations() {
				var formData = doMapping();

				assertThat(formData.getNumberOfRepresentations()).isEqualTo(2);
			}
		}

		@DisplayName("with empty attachments")
		@Nested
		class TestWithEmptyAttachments {
			@DisplayName("should throw technical exception")
			@Test
			void shouldThrowTechnicalException() {
				depositData = DepositDataTestFactory.create(Collections.emptyList());

				assertThatThrownBy(TestMapToFormData.this::doMapping)
						.isInstanceOf(TechnicalException.class);
			}
		}

		private FormData doMapping() {
			return mapper.mapToFormData(depositData);
		}

		private List<String> expectedAttachmentIds(List<Attachment> attachmentList) {
			return attachmentList.stream().map(attachment -> attachment.id).toList();
		}

	}

	@DisplayName("map attachment to incoming file")
	@Nested
	class TestMapAttachmentToIncomingFile {
		private Attachment attachment;

		@BeforeEach
		void mock() {
			attachment = AttachmentTestFactory.createXmlDaten();
		}

		@DisplayName("should have ID")
		@Test
		void shouldHaveId() {
			var incomingFile = doMapping();

			assertThat(incomingFile.getId()).isNotNull();
		}

		@DisplayName("should have vendor ID")
		@Test
		void shouldHaveVendorId() {
			var incomingFile = doMapping();

			assertThat(incomingFile.getVendorId()).isEqualTo(XML_ATTACHMENT_ID);
		}

		@DisplayName("should have name")
		@Test
		void shouldHaveName() {
			var incomingFile = doMapping();

			assertThat(incomingFile.getName()).isEqualTo(XML_NAME);
		}

		@DisplayName("should have content type")
		@Test
		void shouldHaveContentType() {
			var incomingFile = doMapping();

			assertThat(incomingFile.getName()).isEqualTo(XML_NAME);
		}

		@DisplayName("should have size")
		@Test
		void shouldHaveSize() {
			var incomingFile = doMapping();

			assertThat(incomingFile.getSize()).isEqualTo(XML_CONTENT.length);
		}

		@DisplayName("should have file with content")
		@SneakyThrows
		@Test
		void shouldHaveFileWithContent() {
			var incomingFile = doMapping();

			var content = FileUtils.readFileToString(incomingFile.getFile(), Charset.defaultCharset());
			assertThat(content).isEqualTo(XML_CONTENT_STRING);
		}

		private IncomingFile doMapping() {
			return mapper.mapAttachmentToIncomingFile(attachment);
		}
	}

}
