package de.ozgcloud.eingang.semantik.enginebased.dfoerdermittel;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;

import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.dataformat.xml.XmlMapper;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.ServiceKonto;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import de.ozgcloud.eingang.semantik.enginebased.EngineBasedSemantikAdapter;
import de.ozgcloud.eingang.semantik.enginebased.ServiceKontoBuildHelper;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
public class DFoerdermittelEngineBasedSemantikAdapter implements EngineBasedSemantikAdapter {

	private static final String FACHNACHRICHT_SUFFIX = "Fachnachricht.xml";
	private static final Predicate<IncomingFile> IS_FACHNACHRICHT = inFile -> StringUtils.endsWith(inFile.getName(), FACHNACHRICHT_SUFFIX);

	private static final String KEY_FACHNACHRICHT = "Fachnachricht";
	private static final String KEY_POSTFACH_ID = "InboxReference";
	private static final String KEY_ORGANISATIONS_EINHEIT_ID = "MetaText1";

	@Autowired
	private ServiceKontoBuildHelper serviceKontoHelper;

	@Override
	public FormData parseFormData(FormData formData) {
		return formData.getRepresentations().stream().filter(IS_FACHNACHRICHT).findAny()
				.map(inFile -> parseFachnachricht(formData, inFile))
				.map(this::processFachnachricht)
				.orElse(formData);
	}

	FormData processFachnachricht(FormData formData) {
		@SuppressWarnings("unchecked")
		Map<String, Object> fachnachricht = (Map<String, Object>) MapUtils.getMap(formData.getFormData(), KEY_FACHNACHRICHT,
				Collections.<String, Object>emptyMap());

		var extendedFormData = addFormName(formData);
		extendedFormData = addFormEngineName(extendedFormData);
		extendedFormData = addServiceKonto(extendedFormData, fachnachricht);
		return addOrganisationsEinheitId(extendedFormData, fachnachricht);
	}

	FormData addServiceKonto(FormData formData, Map<String, Object> fachnachricht) {
		return Optional.ofNullable((String) fachnachricht.get(KEY_POSTFACH_ID))
				.map(this::extractPrefix)
				.map(this::createServiceKonto)
				.map(serviceKonto -> formData.getHeader().toBuilder().serviceKonto(serviceKonto).build())
				.map(header -> formData.toBuilder().header(header).build())
				.orElse(formData);
	}

	FormData addOrganisationsEinheitId(FormData formData, Map<String, Object> fachnachricht) {
		return Optional.ofNullable((String) fachnachricht.get(KEY_ORGANISATIONS_EINHEIT_ID))
				.map(orgaId -> addOrganisationsEinheitId(orgaId, formData.getZustaendigeStelle()))
				.map(zustStelle -> formData.toBuilder().zustaendigeStelle(zustStelle).build())
				.orElse(formData);
	}

	private ZustaendigeStelle addOrganisationsEinheitId(String orgaId, ZustaendigeStelle zustaendigeStelle) {
		ZustaendigeStelle.ZustaendigeStelleBuilder zustaendigeStelleBuilder;
		if (Objects.isNull(zustaendigeStelle)) {
			zustaendigeStelleBuilder = ZustaendigeStelle.builder();
		} else {
			zustaendigeStelleBuilder = zustaendigeStelle.toBuilder();
		}

		return zustaendigeStelleBuilder.organisationseinheitenId(orgaId).build();
	}

	FormData addFormName(FormData formData) {
		return formData.toBuilder()
				.header(formData.getHeader().toBuilder()
						// TODO replace formName with actual name <Title> from Fachnachricht (KOP-2239)
						.formName("dFördermittelantrag")
						.build()
				)
				.build();
	}

	FormData addFormEngineName(FormData formData) {
		return formData.toBuilder()
				.header(formData.getHeader().toBuilder()
						.formEngineName("dFördermittelantrag")
						.build()
				)
				.build();
	}

	String extractPrefix(@NonNull String postfachId) {
		return postfachId.substring(postfachId.lastIndexOf("/") + 1);
	}

	private ServiceKonto createServiceKonto(String postfachId) {
		return serviceKontoHelper.buildOsiServiceKonto(postfachId);
	}

	FormData parseFachnachricht(FormData formData, IncomingFile fachnachrichtFile) {
		var fachnachrichtData = extractFormDataFormXML(fachnachrichtFile.getContentStream());

		if (MapUtils.isNotEmpty(fachnachrichtData)) {
			var editable = new HashMap<>(formData.getFormData());
			editable.put(KEY_FACHNACHRICHT, fachnachrichtData);
			return formData.toBuilder().formData(Collections.unmodifiableMap(editable)).build();
		}

		return formData;
	}

	@SuppressWarnings("unchecked")
	Map<String, Object> extractFormDataFormXML(InputStream xmlFileStream) {

		XmlMapper xmlMapper = new XmlMapper();
		try {
			return xmlMapper.readValue(xmlFileStream, Map.class);
		} catch (IOException e) {
			LOG.error("Error reading xml fachnachricht.", e);
		}
		return Collections.emptyMap();
	}

	@Override
	public boolean isResponsible(FormData formData) {
		return formData.getRepresentations().stream().anyMatch(IS_FACHNACHRICHT);
	}

}
