package de.ozgcloud.eingang.xdomea;

import static de.ozgcloud.eingang.xdomea.XdomeaMessageDataMapper.*;
import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;

class XdomeaMessageDataMapperTest {

	@Spy
	@InjectMocks
	XdomeaMessageDataMapper fileClassifier;

	@Mock
	private XdomeaXMLValueReader valueReader;

	@DisplayName("find Xdomea XML file")
	@Nested
	class TestFindXdomeaXmlFile {

		private static final String FILE_NAME_WITHOUT_SUFFIX = "some-file-name.xml";
		private static final String FILE_NAME_WITH_SUFFIX = "some-file-name" + XDOMEA_0201_XML_FILE_NAME_SUFFIX;

		@DisplayName("should throw if not found")
		@Test
		void shouldThrowIfNotFound() {
			var incomingFilesWithout = List.of(IncomingFileTestFactory.createWithName(FILE_NAME_WITHOUT_SUFFIX));

			assertThatThrownBy(() -> fileClassifier.findXdomeaXMLFile(incomingFilesWithout))
					.isInstanceOf(TechnicalException.class);
		}

		@DisplayName("should return if found")
		@Test
		void shouldReturnIfFound() {
			var targetIncomingFile = IncomingFileTestFactory.createWithName(FILE_NAME_WITH_SUFFIX);
			var incomingFilesWith = List.of(
					IncomingFileTestFactory.createWithName(FILE_NAME_WITHOUT_SUFFIX),
					targetIncomingFile
			);

			var primaryRepresentation = fileClassifier.findXdomeaXMLFile(incomingFilesWith);

			assertThat(primaryRepresentation).isEqualTo(targetIncomingFile);
		}
	}

	@DisplayName("find primary representation name")
	@Nested
	class TestFindPrimaryRepresentationName {
		private static final String FILE_NAME_WITHOUT_XML_SUFFIX = "some-file-name.pdf";
		private static final String FILE_NAME_WITHOUT_XML_SUFFIX2 = "some-file-name.xml.pdf";
		private static final String FILE_NAME_WITH_XML_SUFFIX = "some-file-name.xml";
		private static final String FILE_NAME_WITH_XML_SUFFIX2 = "some-file-name.pdf.xml";

		@DisplayName("should throw if no xml suffix")
		@Test
		void shouldThrowIfNoXmlSuffix() {
			var listWithoutSuffix = List.of(FILE_NAME_WITHOUT_XML_SUFFIX, FILE_NAME_WITHOUT_XML_SUFFIX2);

			assertThrows(TechnicalException.class, () -> fileClassifier.findPrimaryRepresentationName(listWithoutSuffix));
		}

		@DisplayName("should return first with xml suffix")
		@Test
		void shouldReturnFirstWithXmlSuffix() {
			var listWithSuffix = List.of(FILE_NAME_WITHOUT_XML_SUFFIX, FILE_NAME_WITH_XML_SUFFIX, FILE_NAME_WITHOUT_XML_SUFFIX2,
					FILE_NAME_WITH_XML_SUFFIX2);

			var fileName = fileClassifier.findPrimaryRepresentationName(listWithSuffix);

			assertThat(fileName).isEqualTo(FILE_NAME_WITH_XML_SUFFIX);
		}
	}

	@DisplayName("classify attachments and representations")
	@Nested
	class TestClassifyAttachmentsAndRepresentations {

		private static final String XDOMEA_XML_NAME = "xdomea.xml";
		private static final String REPR_XML_NAME = "repr.xml";
		private static final String REPR_PDF_NAME = "repr.pdf";
		private static final String ATTATCHMENT_XML_NAME = "att.xml";
		private static final String ATTATCHMENT_PNG_NAME = "att.png";
		private static final String ATTATCHMENT_PDF_NAME = "att.pdf";

		@Mock
		private IncomingFile xdomeaXMLFile;

		private List<IncomingFile> incomingFileList;

		@BeforeEach
		void mock() {
			incomingFileList = Stream.of(
							XDOMEA_XML_NAME,
							REPR_XML_NAME,
							REPR_PDF_NAME,
							ATTATCHMENT_XML_NAME,
							ATTATCHMENT_PNG_NAME,
							ATTATCHMENT_PDF_NAME
					)
					.map(IncomingFileTestFactory::createWithName)
					.toList();
			doReturn(xdomeaXMLFile).when(fileClassifier).findXdomeaXMLFile(incomingFileList);

			when(xdomeaXMLFile.getName()).thenReturn(XDOMEA_XML_NAME);

			var representationFileNames = List.of(REPR_PDF_NAME, REPR_XML_NAME);
			when(valueReader.readRepresentationFileNames(xdomeaXMLFile)).thenReturn(representationFileNames);
			doReturn(REPR_XML_NAME).when(fileClassifier).findPrimaryRepresentationName(representationFileNames);
		}

		@DisplayName("should contain xdomea metadata file")
		@Test
		void shouldContainXdomeaMetadataFile() {
			var classification = doClassify();

			var primaryDocument = classification.metadataFile();

			assertThat(primaryDocument.getName()).isEqualTo(XDOMEA_XML_NAME);
		}

		@DisplayName("should contain representations")
		@Test
		void shouldContainRepresentations() {
			var classification = doClassify();

			var resultRepresentationFileNames = classification.representations().stream()
					.map(IncomingFile::getName)
					.toList();
			// Expect that the primary representation xml file is move to the first position
			assertThat(resultRepresentationFileNames).isEqualTo(List.of(REPR_XML_NAME, REPR_PDF_NAME));
		}

		@DisplayName("should contain attachments")
		@Test
		void shouldContainAttachments() {
			var classification = doClassify();

			var resultAttachmentFileNames = classification.attachments()
					.getFirst()
					.getFiles().stream()
					.map(IncomingFile::getName)
					.toList();
			assertThat(resultAttachmentFileNames).isEqualTo(List.of(ATTATCHMENT_XML_NAME, ATTATCHMENT_PNG_NAME, ATTATCHMENT_PDF_NAME));
		}

		@DisplayName("should have attachment group name")
		@Test
		void shouldHaveAttachmentGroupName() {
			var classification = doClassify();

			var resultAttachmentGroupName = classification.attachments()
					.getFirst()
					.getName();
			assertThat(resultAttachmentGroupName).isEqualTo(ATTACHMENT_GROUP_NAME);
		}

		private XdomeaMessageData doClassify() {
			return fileClassifier.mapIncomingFilesToXdomeaMessageData(incomingFileList);
		}
	}

}
