package de.ozgcloud.eingang.xta;

import java.io.IOException;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import de.ozgcloud.common.test.TestUtils;
import okhttp3.mockwebserver.MockResponse;

public class MockResponseTestFactory {

	private static final String UUID_BOUNDARY = "uuid:4403149a-87eb-4bb4-b885-472816010e04+id=12707";
	// The MIME Multipart/Related Content-type (See https://www.ietf.org/rfc/rfc2387.txt)
	private static final String MULTIPART_RELATED_CONTENT_TYPE = "multipart/related; type=\"application/xop+xml\";start=\"<http://tempuri.org/0>\";boundary=\"%s\";start-info=\"application/soap+xml\"".formatted(
			UUID_BOUNDARY);
	private static final String INCLUDE_URL = "http://tempuri.org/1/638485294711394846";

	private static final Map<String, String> ENVELOPE_HEADERS = Map.of(
			"Content-ID", "<http://tempuri.org/0>",
			"Content-Transfer-Encoding", "8bit",
			"Content-Type", "application/xop+xml;charset=utf-8;type=\"application/soap+xml\""
	);

	public static final Map<String, String> MESSAGE_TYPE_BY_ATTACHMENT_FILENAME = Map.of(
			"mantelantrag_without_anlage.zip", "Geschaeftsgang.Geschaeftsgang.0201",
			"dfoerdermittel_without_anlage.zip", "Geschaeftsgang.Geschaeftsgang.0201",
			"versammlungsanzeige.xml", "fim.S17000652.17000652001004",
			"mantelantrag_with_anlage.zip", "Geschaeftsgang.Geschaeftsgang.0201",
			"dfoerdermittel_with_anlage.zip", "Geschaeftsgang.Geschaeftsgang.0201",
			"waffenschein.zip", "Geschaeftsgang.Geschaeftsgang.0201"
	);

	private static final Map<String, String> MESSAGE_ID_BY_ATTACHMENT_FILENAME = MESSAGE_TYPE_BY_ATTACHMENT_FILENAME
			.keySet().stream()
			.collect(Collectors.toMap(name -> name, name -> generateMessageID()));

	private static String generateMessageID() {
		return "urn:de:xta:messageid:dataport_xta_210:%s".formatted(UUID.randomUUID().toString());
	}

	private static final Map<String, String> METADATA_NAME_ATTACHMENT_FILENAME = Map.of(
			"mantelantrag_without_anlage.zip", "afm",
			"dfoerdermittel_without_anlage.zip", "dfoerdermittel",
			"versammlungsanzeige.xml", "versammlungsanzeige",
			"mantelantrag_with_anlage.zip", "afm",
			"dfoerdermittel_with_anlage.zip", "dfoerdermittel",
			"waffenschein.zip", "afm"
	);

	public static MockResponse create() {
		return new MockResponse();
	}

	public static MockResponse createEmptyGetStatusListResponse() {
		var envelopeXMLString = TestUtils.loadTextFile(
				"mock-responses/getStatusList/envelope.template.xml",
				"0",
				""
		);

		var body = combineParts(
				createPart(
						ENVELOPE_HEADERS,
						envelopeXMLString
				)
		);

		return new MockResponse()
				.addHeader("Content-Type", MULTIPART_RELATED_CONTENT_TYPE)
				.setBody(body);
	}

	public static MockResponse createGetStatusListResponse(List<String> xtaAttachmentFileNames) {
		var messageMetaDataEntriesString = xtaAttachmentFileNames.stream()
				.map(MockResponseTestFactory::createMessageMetadataXml)
				.collect(Collectors.joining());
		var envelopeXMLString = TestUtils.loadTextFile(
				"mock-responses/getStatusList/envelope.template.xml",
				String.valueOf(xtaAttachmentFileNames.size()),
				messageMetaDataEntriesString
		);

		var body = combineParts(
				createPart(
						ENVELOPE_HEADERS,
						envelopeXMLString
				)
		);

		return new MockResponse()
				.addHeader("Content-Type", MULTIPART_RELATED_CONTENT_TYPE)
				.setBody(body);
	}

	public static MockResponse createGetMessageResponse(String xtaAttachmentFileName) {
		var messageType = Objects.requireNonNull(MESSAGE_TYPE_BY_ATTACHMENT_FILENAME.get(xtaAttachmentFileName));
		var metadataMessageXml = createMessageMetadataXml(xtaAttachmentFileName);

		var envelopeXMLString = TestUtils.loadTextFile(
				"mock-responses/getMessage/%s/envelope.template.xml".formatted(
						messageType
				),
				metadataMessageXml,
				xtaAttachmentFileName,
				getFileSize(getAttachmentFilePath(xtaAttachmentFileName)),
				INCLUDE_URL
		);

		var body = combineParts(
				createPart(
						ENVELOPE_HEADERS,
						envelopeXMLString
				),
				createPart(
						Map.of(
								"Content-ID", "<%s>".formatted(INCLUDE_URL),
								"Content-Transfer-Encoding", "base64",
								"Content-Type", "application/octet-stream"
						),
						loadFileAsBase64(
								getAttachmentFilePath(xtaAttachmentFileName)
						)
				)
		);

		return new MockResponse()
				.addHeader("Content-Type", MULTIPART_RELATED_CONTENT_TYPE)
				.setBody(body);
	}

	private static String getMessageType(String xtaAttachmentFileName) {
		return Objects.requireNonNull(
				MESSAGE_TYPE_BY_ATTACHMENT_FILENAME.get(xtaAttachmentFileName),
				"Xta-message type for '%s' has to be configured!".formatted(xtaAttachmentFileName)
		);
	}

	private static String createMessageMetadataXml(String xtaAttachmentFileName) {
		return TestUtils.loadTextFile(
				"mock-responses/getStatusList/MessageMetaData/%s/%s.template.xml".formatted(
						getMessageType(xtaAttachmentFileName),
						Objects.requireNonNull(
								METADATA_NAME_ATTACHMENT_FILENAME.get(xtaAttachmentFileName),
								"Xta-message metadata-name for '%s' has to be configured!".formatted(xtaAttachmentFileName)
						)
				), Objects.requireNonNull(MESSAGE_ID_BY_ATTACHMENT_FILENAME.get(xtaAttachmentFileName)),
				getFileSize(getAttachmentFilePath(xtaAttachmentFileName))
		);
	}

	private static String getFileSize(String filePath) {
		try (var inputStream = TestUtils.loadFile(filePath)) {
			return String.valueOf(inputStream.readAllBytes().length);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private static String getAttachmentFilePath(String xtaAttachmentFileName) {
		return "mock-responses/getMessage/%s/%s".formatted(getMessageType(xtaAttachmentFileName), xtaAttachmentFileName);
	}

	private static String loadFileAsBase64(String fileName) {
		try (var attachmentFile = TestUtils.loadFile(fileName)) {
			return new String(Base64.getEncoder().encode(attachmentFile.readAllBytes()));
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private static String combineParts(String... parts) {
		return Stream.concat(
						Arrays.stream(parts).map(part -> "--%s\n%s\n".formatted(UUID_BOUNDARY, part)),
						Stream.of("--%s--".formatted(UUID_BOUNDARY)))
				.collect(Collectors.joining());
	}

	private static String createPart(Map<String, String> headers, String content) {
		return String.join("\n",
				headers.entrySet().stream().map(kv -> "%s: %s\n".formatted(kv.getKey(), kv.getValue())).collect(Collectors.joining()), content);
	}

	public static MockResponse createCloseResponse() {
		return new MockResponse();
	}
}
