package de.ozgcloud.eingang.xta;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;

import java.net.InetAddress;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import okhttp3.mockwebserver.MockWebServer;

@Configuration
public class XtaMockServerConfiguration {

	@Bean
	XtaMocker xtaMockServer(XtaProperties properties) {
		return new XtaMocker(properties);
	}

	@RequiredArgsConstructor
	@Getter
	public static class XtaMocker {
		private final XtaProperties properties;

		private MockWebServer nachrichtenBrokerMock;
		private List<String> filteredAttachmentFileNames;

		@SneakyThrows
		public void setupServer(List<String> xtaAttachmentFileNames, boolean expectCloseMessages) {
			filteredAttachmentFileNames = xtaAttachmentFileNames.stream()
					.filter(name -> Objects.requireNonNull(
									MockResponseTestFactory.MESSAGE_TYPE_BY_ATTACHMENT_FILENAME.get(name),
									"Xta-attachment name '%s' has to be configured!".formatted(name)
							)
							.equals("Geschaeftsgang.Geschaeftsgang.0201"))
					.toList();

			startServer();

			nachrichtenBrokerMock.enqueue(MockResponseTestFactory.createGetStatusListResponse(xtaAttachmentFileNames));
			filteredAttachmentFileNames.forEach(xtaAttachmentFileName -> {
						nachrichtenBrokerMock.enqueue(MockResponseTestFactory.createGetMessageResponse(xtaAttachmentFileName));
						if (expectCloseMessages) {
							nachrichtenBrokerMock.enqueue(MockResponseTestFactory.createCloseResponse());
						}
					}
			);
			nachrichtenBrokerMock.enqueue(MockResponseTestFactory.createEmptyGetStatusListResponse());
		}

		@SneakyThrows
		private void startServer() {
			nachrichtenBrokerMock = new MockWebServer();

			nachrichtenBrokerMock.start(InetAddress.getByName("127.0.0.1"), 0);
			var serverProperties = properties.getServer();
			serverProperties.setAddress(nachrichtenBrokerMock.getHostName() + ":" + nachrichtenBrokerMock.getPort());
			serverProperties.setProtocol("http");
		}

		@SneakyThrows
		public void teardownServer() {
			if (nachrichtenBrokerMock == null) {
				throw new RuntimeException("No server has been set up to tear down!");
			}
			try {
				var expectedCallCount = filteredAttachmentFileNames.size() + 2;

				for (var i = 0; i < expectedCallCount; i++) {
					var request = nachrichtenBrokerMock.takeRequest(1, TimeUnit.SECONDS);
					assertNotNull(request, "should have request index %d".formatted(i));
					assertThat(request.getPath()).isEqualTo("/MB_XTA-WS/XTA210msgBoxPort.svc");
				}
			} finally {
				stopServer();
			}
		}

		@SneakyThrows
		private void stopServer() {
			nachrichtenBrokerMock.shutdown();
			nachrichtenBrokerMock = null;
		}

	}
}
